import json

import requests

from deploy.domain.models.cloud_connexa_models import OAuthResponse, RouteRequest, NetworkCreateRequest
from deploy.domain.file_path import FilePath
from deploy.domain.models.tenant_models import TenantCloudConnexa


class CloudConnexa:
    _config_file_path: str = f"{FilePath().get_path(__file__)}/../config_files/cloud_connexa_config.json"
    _token: str

    def __init__(self):
        self.config_file: dict = self.get_json_from_config_file()

    @classmethod
    def get_token(cls) -> str:
        config = cls.get_json_from_config_file()

        url = (f"https://{config.get('cloud_id')}.api.openvpn.com/api/beta/oauth/token?client_id="
               f"{config.get('client_id')}"
               f"&client_secret={config.get('client_secret')}")

        headers = {
            'Accept': 'application/json',
        }

        response = requests.request("POST", url, headers=headers)
        response.raise_for_status()
        oauth_response: OAuthResponse = OAuthResponse(**response.json())
        print(response.json()["access_token"])
        return response.json()["access_token"]

    @classmethod
    def get_json_from_config_file(cls) -> dict:
        with open(cls._config_file_path, "r") as file:
            return json.load(file)

    def configure_client_side(self, tenant: TenantCloudConnexa) -> str:
        url = f"https://{self.config_file.get('cloud_id')}.api.openvpn.com/api/beta/networks"
        headers = {
            'Authorization': f'Bearer {self.get_token()}',
        }

        # create networks, connectors and allow client networks
        routes: list[RouteRequest] = []
        for network in tenant.network_config.allowed_networks:
            routes.append(RouteRequest(allowEmbeddedIp='false', description="routes", value=network))
        connectors: list = [
            {
                "description": f"{tenant.name} Client Network Side",
                "name": f"{tenant.name}Client",
                "vpnRegionId": "es-mad"
            }
        ]

        payload = NetworkCreateRequest(routes=routes, connectors=connectors,
                                       description=f"{tenant.name} Network", egress='false',
                                       internetAccess="LOCAL", name=f"{tenant.name}ClientNetwork")

        client_network_response = requests.request("POST", url, headers=headers, json=payload.model_dump())

        client_connector_id: str = client_network_response.json()["connectors"][0]["id"]
        client_network_id: str = client_network_response.json()["id"]

        # Añado IP service de conector lado cliente
        url = f"https://{self.config_file.get('cloud_id')}.api.openvpn.com/api/beta/ip-services?networkItemId={client_network_id}&networkItemType=NETWORK"
        payload = {
            "config": {
                "serviceTypes": [
                    "ANY"
                ]
            },
            "description": "client Connector IP Network",
            "name": "ConnectorNetwork",
            "routes": [
                {
                    "description": "this is a client connector IP",
                    "value": f"{tenant.network_config.ip_cidr}"
                }
            ],
            "type": "IP_SOURCE"
        }

        ip_add_service_response = requests.request("POST", url, headers=headers, json=payload)
        print(ip_add_service_response.json())

        url = f"https://{self.config_file.get('cloud_id')}.api.openvpn.com/api/beta/connectors/{client_connector_id}/profile/encrypt"

        r = requests.request("POST", url, headers=headers).text
        print(f"respuesta de configure_client_side{r}")
        return r

    def configure_server_side(self, tenant_name: str, server_private_ip: str, server_private_network: str) -> str:
        url = f"https://{self.config_file.get('cloud_id')}.api.openvpn.com/api/beta/networks"

        headers = {
            'Authorization': f'Bearer {self.get_token()}',
        }

        server_routes: list[RouteRequest] = []
        print(f"SERVER_PRIVATE_NETWORK:{server_private_network}")
        server_routes.append(RouteRequest(allowEmbeddedIp='false', description="Server IP Route",
                                                          value=f"{server_private_network}"))

        print(server_routes[0].model_dump())

        server_connectors: list = [
            {
                "description": f"{tenant_name} Server Side",
                "name": f"{tenant_name}Server",
                "vpnRegionId": "es-mad"
            }
        ]

        payload = NetworkCreateRequest(routes=server_routes, connectors=server_connectors,
                                       description=f"{tenant_name} Server Network", egress='false',
                                       internetAccess="LOCAL", name=f"{tenant_name}ServerNetwork")

        server_network_response = requests.request("POST", url, headers=headers, json=payload.model_dump())

        print(server_network_response.json())

        # obtain networks ids
        server_network_id: str = server_network_response.json()["id"]
        server_connector_id: str = server_network_response.json()["connectors"][0]["id"]


        # Añado IP service de conector lado servidor
        url = f"https://{self.config_file.get('cloud_id')}.api.openvpn.com/api/beta/ip-services?networkItemId={server_network_id}&networkItemType=NETWORK"
        payload = {
            "config": {
                "serviceTypes": [
                    "ANY"
                ]
            },
            "description": "Server IP",
            "name": "serverIP",
            "routes": [
                {
                    "description": "this is a Server IP",
                    "value": f"{server_private_ip}/32"
                }
            ],
            "type": "IP_SOURCE"
        }
        print(payload)

        ip_add_service_response = requests.request("POST", url, headers=headers, json=payload)
        print(ip_add_service_response.json())

        url = f"https://{self.config_file.get('cloud_id')}.api.openvpn.com/api/beta/networks/{server_network_id}/routes"
        resp = requests.request("GET", url, headers=headers)
        id_route_to_update = resp.json()[0]["id"]

        url = f"https://{self.config_file.get('cloud_id')}.api.openvpn.com/api/beta/networks/{server_network_id}/routes/{id_route_to_update}"
        payload = {
            "description": "Restricted to Server IP",
            "value": f"{server_private_ip}/32"
        }
        print(payload)
        update_route_server_response = requests.request("PUT", url, headers=headers, json=payload)
        print(update_route_server_response.json())

        url = f"https://{self.config_file.get('cloud_id')}.api.openvpn.com/api/beta/connectors/{server_connector_id}/profile/encrypt"
        r = requests.request("POST", url, headers=headers).text
        print(r)
        return r

