import paramiko
from colorama import Fore, Style
from paramiko.client import SSHClient
from pydantic import BaseModel

from deploy.domain.file_path import FilePath
from deploy.domain.ssh_interface import ISSH


class Response(BaseModel):
    stdout: str
    stderr: str


class SSHCommandResponse(BaseModel):
    command: str
    response: Response


class SSHCommandResponses(BaseModel):
    responses: list[SSHCommandResponse]


class GoogleCloudSSH(ISSH):

    def __init__(self, hostname: str, username: str, password: str = "guest", port: int = 22,
                 private_key_path: str = f"{FilePath().get_path(__file__)}/../config_files/id_rsa"):
        self.hostname = hostname
        self.username = username
        self.password = password
        self.port = port
        self.private_key_path = private_key_path
        self.private_key = paramiko.RSAKey(filename=self.private_key_path)
        self.last_responses = None

    def _connect(self) -> SSHClient:
        try:
            ssh_connection = paramiko.SSHClient()
            ssh_connection.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            ssh_connection.connect(self.hostname, self.port, self.username, pkey=self.private_key)
            return ssh_connection

        except TimeoutError as e:
            print(f"ERROR: {e}")

    def send_cmd(self, cmd: list[str]) -> dict:
        ssh_connection = self._connect()
        output: list = []
        for command_to_exec in cmd:
            print(f"cmd: {command_to_exec}")
            stdin, stdout, stderr = ssh_connection.exec_command(command_to_exec)
            stdin.flush()
            response = Response(stdout=stdout.read(), stderr=stderr.read())
            print(stdout.read().decode())
            print(stderr.read().decode())
            command_response = SSHCommandResponse(command=command_to_exec, response=response)
            output.append(command_response)
        self._disconnect(ssh_connection)
        self.last_responses = {"responses": output}
        return self.last_responses

    @staticmethod
    def _disconnect(ssh_connection: SSHClient):
        ssh_connection.close()

    def parse_to_str(self):
        for response in self.last_responses["responses"]:
            command = response.model_dump()["command"]
            stdout = response.model_dump()['response']['stdout']
            stderr = response.model_dump()['response']['stderr']

            print(f"Command: {Fore.BLUE}{command}{Style.RESET_ALL}")
            if stdout:
                print(f"  {Fore.GREEN}stdout: {stdout.strip()}{Style.RESET_ALL}")
            if stderr:
                print(f"  {Fore.RED}stderr: {stderr.strip()}{Style.RESET_ALL}")

            print("*" * 80)
            print("\n")

    def put(self, source: str, destination: str):
        sftp = self._connect().open_sftp()
        sftp.put(f"{source}", f"{destination}")
        sftp.close()
