import json
import os
from time import sleep
import paramiko
import yaml
import resend
from deploy.domain.cloud_connexa import CloudConnexa
from deploy.domain.file_path import FilePath
from deploy.domain.models.tenant_models import TenantCloudConnexa
from deploy.infraestructure.adapters.deploy_platforms.google_cloud_vm import GoogleCloudVM
from deploy.infraestructure.gateways.network_connector_interface import INetworkConnector


class CloudConnexaNetworkConnector(INetworkConnector):

    def __init__(self):
        self.ssh = paramiko.SSHClient()
        self._config_file_path: str = f"{FilePath().get_path(__file__)}/../../../../config_files/gcloud_ssh_config.json"
        self._config_ssh_connection = None
        with open(self._config_file_path, "r") as file:
            self._config_ssh_connection = json.load(file)
        self.username = self._config_ssh_connection.get("username")
        self.hostname = self._config_ssh_connection.get("hostname")
        self.port = self._config_ssh_connection.get("port")
        self.cloud_connexa: CloudConnexa = CloudConnexa()

    def get_connector(self, tenant: TenantCloudConnexa) -> dict:

        virtual_machine = GoogleCloudVM()
        virtual_machine.deploy(tenant.name)
        sleep(60)

        public_ip: str = virtual_machine.google_cloud.get_public_ip()
        private_ip: str = virtual_machine.google_cloud.get_private_ip()
        private_network: str = virtual_machine.google_cloud.get_subnetwork()

        # Retrieve Cloud Connexa Token configuration
        client_token: str = self.cloud_connexa.configure_client_side(tenant)
        server_token: str = self.cloud_connexa.configure_server_side(tenant.name, private_ip, private_network)

        # Configure Server VM connector
        self.configure_server(public_ip, server_token)
        self.configure_client(client_token, tenant.network_config.ip_cidr, tenant.network_config.gateway)

        # Send email
        self.send_email(tenant, public_ip)

        return {
            "message": "Service provisioning In Progress. You will receive an email in a few minutes..."
        }

    @staticmethod
    def send_email(tenant: TenantCloudConnexa, public_ip: str):

        with open(f"{FilePath().get_path(__file__)}/../../../../config_files/resend.json", "r") as file:
            resend_configuration = json.load(file)

        resend.api_key = resend_configuration["apiKey"]

        r = resend.Emails.send({
            "from": f"{resend_configuration['email']}",
            "to": f"{tenant.user.email}",
            "subject": "Network Manager service provisioned",
            "html": f"<p>Congrats! Your service has been provisioned.</p>"
                    f"<p>You can download tour VM connector from http://35.204.159.84/ using this password: P4a44w0rd</p>"
                    f"<p>API server: http://{public_ip}:8000 </p>"
                    f"<p>API documentation: http://{public_ip}:8000/docs</p>"
        })

    def configure_server(self, ip: str, server_token: str):

        self.ssh_connect(ip)
        stdin, stdout, stderr = self.ssh.exec_command('sudo mkdir /var/local/scripts')
        print(stdout.read())
        stdin, stdout, stderr = self.ssh.exec_command('sudo chmod 777 /var/local/scripts')
        print(stdout.read())

        absolute_file_path = os.path.abspath(__file__)
        path, py_filename = os.path.split(absolute_file_path)
        file: str = f"{path}/ubuntu_22_04.sh"
        sftp = self.ssh.open_sftp()
        sftp.put(file, '/var/local/scripts/ubuntu_22_04.sh')

        stdin, stdout, stderr = self.ssh.exec_command('sudo chmod +x /var/local/scripts/ubuntu_22_04.sh')
        print(stdout.read())
        stdin, stdout, stderr = self.ssh.exec_command('sudo sh /var/local/scripts/ubuntu_22_04.sh')
        print(stdout.read())
        stdin, stdout, stderr = self.ssh.exec_command(
            f"echo '{server_token}' | sudo openvpn-connector-setup")
        print(stdout.read())
        self.ssh_disconnect()

    def configure_client(self, client_token: str, ip_cidr: str, gateway: str):
        self.update_ansible_yaml(client_token)
        self.update_network_config_yaml(ip_cidr, gateway)
        file_paths: list[str] = [f"generate_image.yaml", f"inventory.yaml", f"00-installer-config.yaml"]
        self.put(file_paths)
        self.run_ansible_playbooks()

    def update_ansible_yaml(self, client_token: str):

        with open(f"{FilePath().get_path(__file__)}/ansible_playbooks/generate_image_template.yaml", "r",
                  encoding="utf-8") as file:
            config_file = yaml.safe_load(file)

        with open(f"{FilePath().get_path(__file__)}/ansible_playbooks/generate_image.yaml", "w",
                  encoding="utf-8") as file:
            config_file[1]['tasks'][0]['ansible.builtin.shell'][
                'cmd'] = f"echo '{client_token}' | sudo openvpn-connector-setup"
            yaml.safe_dump(config_file, file)

    def update_network_config_yaml(self, ip_cidr: str, gateway: str):

        with open(f"{FilePath().get_path(__file__)}/ansible_playbooks/00-installer-config_template.yaml", "r",
                  encoding="utf-8") as file:
            config_file = yaml.safe_load(file)

        with open(f"{FilePath().get_path(__file__)}/ansible_playbooks/00-installer-config.yaml", "w",
                  encoding="utf-8") as file:
            config_file['network']['ethernets']['enp0s3']['addresses'] = [f"{ip_cidr}"]
            config_file['network']['ethernets']['enp0s3']['gateway4'] = f"{gateway}"

            yaml.safe_dump(config_file, file)

    def run_ansible_playbooks(self):

        self.ssh_connect(self.hostname)
        stdin, stdout, stderr = self.ssh.exec_command(
            'ansible-playbook -i /var/local/images/ansible/inventory.yaml /var/local/images/ansible/generate_image.yaml')
        print(stdout.read())

    def ssh_connect(self, hostname):

        self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        try:
            private_key_path: str = f"{FilePath().get_path(__file__)}/../../../../config_files/id_rsa"
            private_key = paramiko.RSAKey(filename=private_key_path)
            self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            self.ssh.connect(hostname, self.port, self.username, pkey=private_key)


        except Exception as e:
            print(e)

    def put(self, filenames: list[str]):
        private_key_path: str = f"{FilePath().get_path(__file__)}/../../../../config_files/id_rsa"
        private_key = paramiko.RSAKey(filename=private_key_path)
        self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.ssh.connect(self.hostname, self.port, self.username, pkey=private_key)
        sftp = self.ssh.open_sftp()
        print("conectado por SFTP...")
        for filename in filenames:
            sftp.put(f"{FilePath().get_path(__file__)}/ansible_playbooks/{filename}",
                     f"/var/local/images/ansible/{filename}")

        sftp.close()
        self.ssh_disconnect()

    def ssh_disconnect(self):
        self.ssh.close()
